import sys
from dataclasses import dataclass

import dli.column
from dbt.adapters.base.column import Column
from typing import Any, Optional, Dict


@dataclass
class DLIColumn(Column):
    table_name: Optional[str] = None
    table_stats: Optional[Dict[str, Any]] = None
    column_index: Optional[int] = None
    table_type: Optional[str] = None
    table_database: Optional[str] = None
    table_schema: Optional[str] = None
    table_owner: Optional[str] = None

    def literal(self, value) -> str:
        return f"cast({value} as {self.dtype})"

    @property
    def quoted(self) -> str:
        return f"`{self.column}`"

    @property
    def data_type(self) -> str:
        return self.dtype

    @classmethod
    def translate_type(cls, dtype: str) -> str:
        return dtype

    @classmethod
    def numeric_type(cls, dtype: str, precision: Any, scale: Any) -> str:
        if precision is None or scale is None:
            return dtype
        else:
            return "{}({},{})".format("decimal", precision, scale)

    def __repr__(self) -> str:
        return "<DLIColumn {} ({})>".format(self.name, self.data_type)

    def to_column_dict(self, omit_none: bool = True, validate: bool = False):
        return {"column": self.column, "dtype": self.dtype, "table_schema": self.table_schema,
                "table_name": self.table_name, "table_type": self.table_type, "column_index": self.column_index,
                "table_owner": self.table_owner}
