{% macro file_format_clause() %}
  {%- set file_format = config.get('file_format', validator=validation.any[basestring]) -%}
  {%- if file_format is not none and file_format in ('delta', 'hudi') %}
    using {{ file_format }}
  {%- elif file_format is not none -%}
    stored as {{ file_format }}
  {%- else -%}
    stored as parquet
  {%- endif %}
{%- endmacro -%}

{% macro location_clause() %}
  {%- set obs_path = config.get('obs_path', validator=validation.any[basestring]) -%}
  {%- if obs_path is not none %}
    location '{{ obs_path }}'
  {%- endif %}
{%- endmacro -%}

{% macro properties_clause() %}
  {{ return(adapter.dispatch('properties_clause', 'dbt')()) }}
{%- endmacro -%}


{% macro dli__properties_clause() %}
  {%- set properties = config.get('properties', none) -%}
  {%- if properties is not none -%}
    tblproperties(
    {%- for k, v in properties.items() -%}'{{ k }}'='{{ v }}'
    {%- if not loop.last %},{% endif -%}
    {%- endfor -%}
    )
  {%- endif %}
{%- endmacro -%}


{% macro partitioned_by_clause() %}
  {{ return(adapter.dispatch('partitioned_by_clause', 'dbt')()) }}
{%- endmacro -%}


{% macro dli__partitioned_by_clause() %}
  {%- set partitioned_by = config.get('partitioned_by', none) -%}
  {%- if partitioned_by is not none -%}
    partitioned by (
    {%- for item in partitioned_by -%}
      {{ item['col_name'] }} {{ item['data_type'] or 'string' }}{%- if item['comment'] %} comment '{{ item['comment'] }}'{%- endif -%}
      {%- if not loop.last %}, {% endif -%}
    {%- endfor -%}
    )
  {%- endif %}
{%- endmacro -%}


{%- macro dli__current_timestamp() -%}
  current_timestamp()
{%- endmacro -%}


{% macro dli__drop_relation(relation) -%}
  {% call statement('drop_relation', auto_begin=False) -%}
    drop {{ relation.type }} if exists {{ relation }}
  {%- endcall %}
{% endmacro %}

{# tested #}
{% macro dli__create_view_as(relation, sql) -%}
  create or replace view {{ relation }}
  as {{ sql }};
{%- endmacro %}

{# tested #}
{% macro dli__create_table_as(temporary, relation, sql) -%}
  create table if not exists {{ relation }}
    {{ file_format_clause() }}
    {{ location_clause() }}
    {{ properties_clause() }}
  as {{ sql }}
{%- endmacro %}

{# tested #}
{% macro dli__rename_relation(from_relation, to_relation) -%}
  {% set target_name = adapter.quote_as_configured(to_relation.identifier, 'identifier') %}

  {% call statement('rename_relation') -%}
    {% if not from_relation.type %}
      {% do exceptions.raise_database_error("Cannot rename a relation with a blank type: " ~ from_relation.identifier) %}
    {% elif from_relation.type in ('table') %}
        alter table {{ from_relation }} rename to {{ target_name }}
    {% elif from_relation.type == 'view' %}
        alter view {{ from_relation }} rename to {{ target_name }}
    {% else %}
      {% do exceptions.raise_database_error("Unknown type '" ~ from_relation.type ~ "' for relation: " ~ from_relation.identifier) %}
    {% endif %}
  {%- endcall %}
{% endmacro %}

{# tested #}
{% macro show_create_table(relation) %}
  {% call statement('show_create_table', fetch_result=True) -%}
    show create table {{ relation }}
  {%- endcall %}

  {% set result = load_result('show_create_table') %}
  {% do return(result.table[0][0]) %}
{% endmacro %}


{% macro dli__list_tables_without_caching(schema) %}
  {% call statement('list_tables_without_caching', fetch_result=True) -%}
    show tables in {{ schema }}
  {% endcall %}
  {% do return(load_result('list_tables_without_caching').table) %}
{% endmacro %}

{% macro dli__list_views_without_caching(schema) %}
  {% call statement('list_views_without_caching', fetch_result=True) -%}
    show views  in {{ schema }}
    -- show tables  in {{ relation }}
  {% endcall %}
  {% do return(load_result('list_views_without_caching').table) %}
{% endmacro %}

{% macro dli__create_schema(relation) -%}
  {%- call statement('create_schema') -%}
    create database if not exists {{relation}}
  {% endcall %}
{% endmacro %}

{% macro dli__drop_schema(relation) -%}
  {%- call statement('drop_schema') -%}
    drop database if exists {{ relation }} cascade
  {%- endcall -%}
{% endmacro %}

{# use describe extended for more information #}
{% macro dli__get_columns_in_relation(relation) -%}
  {% call statement('get_columns_in_relation', fetch_result=True) %}
    describe formatted {{ relation }}
  {% endcall %}
  {% do return(load_result('get_columns_in_relation').table) %}
{% endmacro %}

{% macro dli__list_schemas(database) -%}
  {% call statement('list_schemas', fetch_result=True, auto_begin=False) %}
    show databases
  {% endcall %}
  {% do return(load_result('list_schemas').table) %}
{% endmacro %}